@@ -2,8 +2,8 @@
 # -*- coding: utf-8 -*-
 
 ###################
-#    This file can add a group
-#    Copyright (C) 2021  Maurice Lambert
+#    This file adds a new group
+#    Copyright (C) 2021, 2022  Maurice Lambert
 
 #    This program is free software: you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
@@ -19,11 +19,13 @@
 #    along with this program.  If not, see <https://www.gnu.org/licenses/>.
 ###################
 
-"""This tool run scripts and display the result in a Web Interface.
+"""
+This tool run scripts and display the result in a Web Interface.
 
-This file can add a group."""
+This file adds a new group.
+"""
 
-__version__ = "0.0.1"
+__version__ = "0.0.2"
 __author__ = "Maurice Lambert"
 __author_email__ = "mauricelambert434@gmail.com"
 __maintainer__ = "Maurice Lambert"
@@ -31,12 +33,13 @@ __maintainer_email__ = "mauricelambert434@gmail.com"
 __description__ = """
 This tool run scripts and display the result in a Web Interface.
 
-This file can add a group."""
+This file adds a new group.
+"""
 __license__ = "GPL-3.0 License"
 __url__ = "https://github.com/mauricelambert/WebScripts"
 
 copyright = """
-WebScripts  Copyright (C) 2021  Maurice Lambert
+WebScripts  Copyright (C) 2021, 2022  Maurice Lambert
 This program comes with ABSOLUTELY NO WARRANTY.
 This is free software, and you are welcome to redistribute it
 under certain conditions.
@@ -47,28 +50,29 @@ __copyright__ = copyright
 __all__ = []
 
 from modules.manage_defaults_databases import add_group, GroupError
-import sys
+from sys import exit, argv, stderr
 
 
-def main() -> None:
-    if len(sys.argv) != 3 and not sys.argv[2].isdigit():
+def main() -> int:
+    if len(argv) != 3 and not argv[2].isdigit():
         print(
-            "USAGE: add_group.py [NAME string required] [ID integer required]"
+            "USAGES: add_group.py [NAME string required] [ID integer required]"
         )
-        sys.exit(1)
+        return 1
 
     try:
-        group = add_group(sys.argv[2], sys.argv[1])
+        group = add_group(argv[2], argv[1])
     except GroupError as error:
-        print(error)
-        sys.exit(2)
+        print(error.__class__.__name__, error, file=stderr)
+        return 2
     except Exception as error:
-        print(error)
-        sys.exit(127)
+        print(error.__class__.__name__, error, file=stderr)
+        return 127
 
     print(f"Group added:\n\t - Name: {group.name}\n\t - ID: {group.ID}")
 
+    return 0
+
 
 if __name__ == "__main__":
-    main()
-    sys.exit(0)
+    exit(main())
