@@ -52,6 +52,7 @@ from WebScripts.commons import (
     TokenCSRF,
     Session,
 )
+from WebScripts import commons
 import WebScripts.commons
 
 
@@ -89,7 +90,7 @@ class TestArgument(TestCase):
         self.assertListEqual([], Argument.get_command("-t", {"value": ""}))
         self.assertListEqual([], Argument.get_command("-t", {"value": False}))
         self.assertListEqual(
-            [], Argument.get_command("-t", {"value": ["", "", ""]})
+            [], Argument.get_command("-t", {"value": ["", "", "", None]})
         )
 
         with self.assertRaises(WebScriptsArgumentError):
@@ -137,9 +138,20 @@ class TestScriptConfig(TestCase):
             ScriptConfig, "get_scripts_from_configuration"
         ) as config_mock:
             config_mock.return_value = {}
-            ScriptConfig.build_scripts_from_configuration(configuration)
 
-        self.assertEqual(len(config_mock.mock_calls), 3)
+            with patch.object(
+                WebScripts.commons, "check_file_permission", return_value=False
+            ):
+                ScriptConfig.build_scripts_from_configuration(configuration)
+                self.assertEqual(len(configuration.configuration_files), 0)
+                self.assertEqual(len(config_mock.mock_calls), 1)
+
+            with patch.object(
+                WebScripts.commons, "check_file_permission", return_value=True
+            ):
+                ScriptConfig.build_scripts_from_configuration(configuration)
+
+        self.assertEqual(len(config_mock.mock_calls), 4)
         self.assertListEqual(
             [
                 path.abspath(file).casefold()
@@ -163,41 +175,48 @@ class TestScriptConfig(TestCase):
             list(configuration.configuration_files.values())[1],
         )
 
-        self.assertEqual(len(config_mock.mock_calls[0].args), 2)
+        self.assertEqual(len(config_mock.mock_calls[1].args), 2)
         self.assertListEqual(
-            config_mock.mock_calls[0].args[0].ini_scripts_config,
+            config_mock.mock_calls[1].args[0].ini_scripts_config,
             ["./unittests_configuration.ini"],
         )
         self.assertListEqual(
-            config_mock.mock_calls[0].args[0].json_scripts_config,
+            config_mock.mock_calls[1].args[0].json_scripts_config,
             ["./unittests_configuration.json"],
         )
 
-        self.assertEqual(len(config_mock.mock_calls[1].args), 2)
-        self.assertDictEqual(
-            config_mock.mock_calls[1].args[0].scripts, {"test.py": "test"}
-        )
-        self.assertDictEqual(config_mock.mock_calls[1].args[0].test, {})
-
         self.assertEqual(len(config_mock.mock_calls[2].args), 2)
         self.assertDictEqual(
-            config_mock.mock_calls[2].args[0].scripts, {"test.sh": "test"}
+            config_mock.mock_calls[2].args[0].scripts, {"test.py": "test"}
         )
         self.assertDictEqual(config_mock.mock_calls[2].args[0].test, {})
 
+        self.assertEqual(len(config_mock.mock_calls[3].args), 2)
+        self.assertDictEqual(
+            config_mock.mock_calls[3].args[0].scripts, {"test.sh": "test"}
+        )
+        self.assertDictEqual(config_mock.mock_calls[3].args[0].test, {})
+
         os.remove("unittests_configuration.ini")
         os.remove("unittests_configuration.json")
 
     def test_get_scripts_from_configuration(self):
         configuration = DefaultNamespace()
         configuration.scripts = {"test.py": "test"}
+        configuration.force_file_permissions = True
 
-        with self.assertRaises(WebScriptsConfigurationError):
+        with self.assertRaises(WebScriptsConfigurationError), patch.object(
+            WebScripts.commons, "check_file_permission", return_value=True
+        ):
             ScriptConfig.get_scripts_from_configuration(configuration, None)
 
         configuration.test = {}
 
-        with patch.object(ScriptConfig, "get_script_path") as mock_config_path:
+        with patch.object(
+            ScriptConfig, "get_script_path"
+        ) as mock_config_path, patch.object(
+            WebScripts.commons, "check_file_permission", return_value=True
+        ):
             mock_config_path.return_value = "/fake/path/test.py"
             script_configs = ScriptConfig.get_scripts_from_configuration(
                 configuration, configuration
@@ -211,12 +230,15 @@ class TestScriptConfig(TestCase):
         configuration = DefaultNamespace()
         configuration.scripts = {"test.py": "test"}
         configuration.test = {"path": "/fake/path/test.py"}
+        configuration.force_file_permissions = True
 
         with patch.object(
             WebScripts.commons, "get_real_path"
         ) as mock_real_path:
             mock_real_path.return_value = "/fake/path/test.py"
-            with self.assertRaises(WebScriptsConfigurationError):
+            with self.assertRaises(WebScriptsConfigurationError), patch.object(
+                WebScripts.commons, "check_file_permission", return_value=True
+            ):
                 ScriptConfig.get_scripts_from_configuration(
                     configuration, configuration
                 )
@@ -226,56 +248,84 @@ class TestScriptConfig(TestCase):
         with open("test.py", "w") as file:
             file.write("# test")
 
-        script_configs = ScriptConfig.get_scripts_from_configuration(
-            configuration, configuration
-        )
-        self.assertTrue(script_configs["test.py"].path_is_defined)
+        with patch.object(
+            WebScripts.commons, "check_file_permission", return_value=False
+        ) as mock_permission:
+            self.assertDictEqual(
+                ScriptConfig.get_scripts_from_configuration(
+                    configuration, configuration
+                ),
+                {},
+            )
+            mock_permission.assert_called_once()
+
+        with patch.object(
+            WebScripts.commons, "check_file_permission", return_value=True
+        ) as mock_permission:
+            script_configs = ScriptConfig.get_scripts_from_configuration(
+                configuration, configuration
+            )
+            mock_permission.assert_called_once()
+            self.assertTrue(script_configs["test.py"].path_is_defined)
 
         os.remove("test.py")
 
     def test_get_Windows_default_script_launcher(self):
-        with patch.object(WebScripts.commons, "system") as mock_system:
-            mock_system.return_value = "Linux"
-            self.assertIsNone(
-                ScriptConfig.get_Windows_default_script_launcher(None)
+        # with patch.object(WebScripts.commons, "system") as mock_system:
+        #     mock_system.return_value = "Linux"
+        WebScripts.commons.IS_WINDOWS = False
+        WebScripts.utils.IS_WINDOWS = False
+        commons.IS_WINDOWS = False
+        self.assertIsNone(
+            ScriptConfig.get_Windows_default_script_launcher(None)
+        )
+
+        # with patch.object(WebScripts.commons, "system") as mock_system:
+        #     mock_system.return_value = "Windows"
+
+        WebScripts.commons.IS_WINDOWS = True
+        WebScripts.utils.IS_WINDOWS = True
+        commons.IS_WINDOWS = True
+        with self.assertRaises(WebScriptsSecurityError):
+            ScriptConfig.get_Windows_default_script_launcher(
+                {"path": "abc.command injection", "name": ""}
             )
 
-        with patch.object(WebScripts.commons, "system") as mock_system:
-            mock_system.return_value = "Windows"
-
-            with self.assertRaises(WebScriptsSecurityError):
+        with patch.object(
+            WebScripts.commons,
+            "Popen",
+            return_value=Mock(
+                returncode=1, communicate=Mock(return_value=("", ""))
+            ),
+        ) as mock_process:
+            self.assertIsNone(
                 ScriptConfig.get_Windows_default_script_launcher(
-                    {"path": "abc.command injection", "name": ""}
+                    {"path": "abc.py", "name": ""}
                 )
+            )
 
-            with patch.object(WebScripts.commons, "Popen") as mock_process:
-                mock_process.return_value = Mock(
-                    returncode=1, communicate=Mock(return_value=("", ""))
-                )
-                self.assertIsNone(
-                    ScriptConfig.get_Windows_default_script_launcher(
-                        {"path": "abc.py", "name": ""}
-                    )
+            mock_process.return_value = Mock(
+                returncode=0, communicate=Mock(return_value=("", ""))
+            )
+            self.assertIsNone(
+                ScriptConfig.get_Windows_default_script_launcher(
+                    {"path": "abc.py", "name": ""}
                 )
+            )
 
-                mock_process.return_value = Mock(
-                    returncode=0, communicate=Mock(return_value=("", ""))
-                )
-                self.assertIsNone(
-                    ScriptConfig.get_Windows_default_script_launcher(
-                        {"path": "abc.py", "name": ""}
-                    )
-                )
+            mock_process.return_value = Mock(
+                returncode=0, communicate=Mock(return_value=('="abc', ""))
+            )
+            self.assertEqual(
+                ScriptConfig.get_Windows_default_script_launcher(
+                    {"path": "abc.py", "name": ""}
+                ),
+                "abc",
+            )
 
-                mock_process.return_value = Mock(
-                    returncode=0, communicate=Mock(return_value=('="abc', ""))
-                )
-                self.assertEqual(
-                    ScriptConfig.get_Windows_default_script_launcher(
-                        {"path": "abc.py", "name": ""}
-                    ),
-                    "abc",
-                )
+        WebScripts.commons.IS_WINDOWS = False
+        WebScripts.utils.IS_WINDOWS = False
+        commons.IS_WINDOWS = False
 
     def test_get_script_path(self):
         server_config = DefaultNamespace()
@@ -516,7 +566,14 @@ class TestCallableFile(TestCase):
             )
             self.assertRegex(
                 headers["Content-Security-Policy"],
-                r"default-src 'self'; form-action 'none'; frame-ancestors 'none'; script-src 'self' 'nonce-[\da-fA-F]{20}",
+                "default-src 'self'; navigate-to 'self'; worker-src "
+                "'none'; style-src-elem 'self'; style-src-attr 'none';"
+                " style-src 'self'; script-src-attr 'none'; object-src"
+                " 'none'; media-src 'none'; manifest-src 'none'; "
+                "frame-ancestors 'none'; connect-src 'self'; font-src"
+                " 'none'; img-src 'self'; base-uri 'none'; child-src"
+                " 'none'; form-action 'none'; script-src 'self' "
+                r"'nonce-[\da-fA-F]{40}' 'require-trusted-types-for'",
             )
             self.assertRegex(
                 content, r"file\.py_test_[\w\d+/]+={0,2}_[\da-fA-F]{20}"
